%==========================================================================
% Asset Pricing function computation
%==========================================================================
clear all;
clc;
load environ_cal_9
Z_STATE   = uint16(Z_STATE);
load H_all
for ii=1:size(MM,2)
    H = H_all{1,ii};
    H
%==========================================================================
% Pricing Kernal
%==========================================================================
  m = beta*Zshocks(Z_STATE(2:T)).^(-gama).*...
      H(T_STATERe(1:T-1)+ZtotalRe*(Z_STATE(2:T)-1))'.^gama;
  uppbnd = std(m)/mean(m);
%==========================================================================
% Conditional Pricing Kernal (history dependent), std(sigma_t(m)/E_t(m))
%==========================================================================
  cm = beta*repmat(Zshocks'.^-gama,ZtotalRe,1).*H.^gama;
  E_cm = sum(cm.*Ztrans(ZlagRe1st,:),2);
  sigma_cm = (sum(cm.^2.*Ztrans(ZlagRe1st,:),2) - E_cm.^2).^(1/2);
  sim_cm = std(sigma_cm(T_STATERe)./E_cm(T_STATERe));
  prob{1} = Ztrans(ZlagRe1st,:);
%==========================================================================
% Risk free interest rate
%==========================================================================
    R_z_final = beta*repmat(Zshocks'.^-gama,ZtotalRe,1).*Ztrans(ZlagRe1st,:).*H.^gama;
    risk_free = sum(R_z_final,2).^(-1);

%==========================================================================
% P/d ratio for aggregate consumption          
%==========================================================================
gapacpd=1;
ac_pd = ones(ZtotalRe,1);
while gapacpd>10^-8;
     for i = 1:ZtotalRe
     ac_pd_new(i,1) = sum((1+ac_pd(ZINDRe(i,:))').*Zshocks'.*R_z_final(i,:),2);
     end
     gapacpd = max(abs(ac_pd - ac_pd_new));
     ac_pd = ac_pd_new;
end
%==========================================================================
% Return for leverage equity claim
%==========================================================================
Return_LC = ((ac_pd(ZINDRe) + 1).*repmat(Zshocks',ZtotalRe,1)./repmat(ac_pd,1,Znum)...
            -L*repmat(risk_free,1,Znum))/(1-L);        
ExReturn_LC = sum(Return_LC.*prob{1},2);
%==========================================================================
% Conditional expectated return for leverage claim       
%==========================================================================
  ExR_LC{1}  = (Return_LC - repmat(risk_free,1,Znum)) + 1;
  CExR_LC(:,1) = sum(ExR_LC{1}.*prob{1},2);
  CStdExR_LC(:,1) = (sum(ExR_LC{1}.^2.*prob{1},2) - CExR_LC(:,1).^2).^(1/2);
  Csharp_LC(:,1) = (CExR_LC(:,1)-1)./CStdExR_LC(:,1);  
%==========================================================================
% Compute the returns (stock, consumption good, bond and risk free rate   
%==========================================================================
Return_LC_tmp = reshape(Return_LC,ZtotalRe*Znum,1);
Return_LC_sim = Return_LC_tmp(T_STATERe(1:T-1)+(Z_STATE(2:T)-1)*ZtotalRe);

return_ac = (Zshocks(Z_STATE(2:T)).*(1+ac_pd(T_STATERe(2:T))))./...
               ac_pd(T_STATERe(1:T-1));
rf           = risk_free(T_STATERe(2:T));
%==========================================================================
% Reporting the Results
%==========================================================================
start  = 500;
ending = 10;

disp('excess return on Leverage claim')
erLc_av  = mean(Return_LC_sim(start+1:T-ending+1)-rf(start:T-ending))*100
erLc_std = std(Return_LC_sim(start+1:T-ending+1)-rf(start:T-ending))*100

disp('std of conditional excess return on Leverage claim')
STDerLc_av    = std(CExR_LC(T_STATERe,1))*100
STDerLc_std   = std(CStdExR_LC(T_STATERe,1))*100
STDerLc_sharp = std(Csharp_LC(T_STATERe,1))*100

disp('risk free interest')
rf_av = (mean(rf(start:T-ending))-1)*100
rf_std = std(rf(start:T-ending))*100

%==========================================================================
% Making tables
%==========================================================================
Allreturn(ii,:) = [std(m)/mean(m); 
                   sim_cm*100;
                   STDerLc_av; 
                   STDerLc_std; 
                   STDerLc_sharp;
                   ];
                    
end

Allreturn = Allreturn';
latex(double(Allreturn),'%.3f');



